<?php defined('SYSPATH') or die('No direct script access.');

class Controller_Student_Subject extends Controller_UserEnvironment
{
    public function action_show() {
        $id = $this->request->param('id');
        $student = $this->user['StudentID'];
        $discipline = Model_Discipline::load($id);
        $rates = Model_Rating::getRates($discipline->ID, $student);

        $twig = Twig::factory('student/subject');
        $twig->User = $this->user;
        $twig->Discipline = self::generateSubjectInfoFrom($discipline);
        $twig->DisciplineMap = self::generateDisciplineMapFrom($rates);

        $this->response->body($twig);
    }

    private static function generateSubjectInfoFrom(Model_Discipline $discipline) {
        $semester = Model_Semesters::getInfo($discipline->semesterID);
        $semester['Num'] = $semester['Num'] == 1 ? 'Осенний' : 'Весенний';

        return array(
            'Title'         => $discipline->subjectName,
            'Control'       => $discipline->type == 'exam' ? 'Экзамен' : 'Зачет',
            'ExamType'      => $discipline->type,
            'LectureCount'  => $discipline->lectures,
            'PracticeCount' => $discipline->practice,
            'LabCount'      => $discipline->labs,
            'Teachers'      => self::getTeachersListOf($discipline),
                            # fixme: DB::GetDisciplineInfo does not contains `DepName`
            'DepName'       => null,
            'Num'           => $semester['Num'],
            'Year'          => $semester['Year'],
        );
    }

    private static function getTeachersListOf(Model_Discipline $discipline) {
        $teachers = Model_Teachers::ofDiscipline($discipline->ID);
        $teachersHandled = array();
        $i = 0;
        foreach ($teachers as $teacher) {
            $i++;
            $teachersHandled[$i] = $teacher['LastName'] . ' ' . $teacher['FirstName'] . ' ';
            if (!empty($teacher['SecondName'])) {
                $teachersHandled[$i] .= $teacher['SecondName'];
            }
        }
        return $teachersHandled;
    }

    private static function generateDisciplineMapFrom(Database_Result $rates) {
        $id = 0;                   # not sure, what it is about :(
        $sRate = 0;                # completely legacy code, baby
        $sMaxRate = 0;
        $extraRate = 0;
        $bonusRate = 0;
        $bonusMaxRate = 0;
        $examRate = 0;
        $examMaxRate = 0;
        $disMap = array();
        $modulesCount = 0;
        $disMap['Extra']['Rate'] = 0;

        foreach ($rates as $row) {
            switch ($row['ModuleType']) {
                case 'exam':
                    $examRate += $row['Rate'];
                    $examMaxRate += (int)$row['MaxRate'];

                    $disMap['Exam'] = array(
                        'Rate'    => (int)$row['Rate'],
                        'MaxRate' => (int)$row['MaxRate'],
                        'Date'    => $row['Date'],
                    );
                    break;

                case 'bonus':
                    $bonusRate += $row['Rate'];
                    $bonusMaxRate += (int)$row['MaxRate'];

                    $disMap['Bonus'] = array(
                        'Description' => $row['Description'],
                        'Rate'        => (int)$row['Rate'],
                        'MaxRate'     => (int)$row['MaxRate'],
                        'Date'        => $row['Date'],
                    );
                    break;

                case 'extra':
                    $extraRate += (int)$row['Rate'];

                    $disMap['Extra']['Description'] = $row['Description'];
                    $disMap['Extra']['Rate'] += (int)$row['Rate'];
                    break;

                default:
                    $sRate += (int)$row['Rate'];
                    $sMaxRate += (int)$row['MaxRate'];

                    if ($row['ModuleID'] != $id) {
                        $id = $row['ModuleID'];
                        $modulesCount++;
                    }

                    $module =& $disMap['Modules'][$modulesCount];

                    if (!isset($module['SubmodulesCount'])) {
                        $module['Rate'] = 0;
                        $module['MaxRate'] = 0;
                        $module['SubmodulesCount'] = 0;
                    }

                    $module['Title']    = $row['ModuleName'];
                    $module['Rate']     += (int)$row['Rate'];
                    $module['MaxRate']  += (int)$row['MaxRate'];
                    $module['SubmodulesCount']++;

                    $module['Submodules'][] = array(
                        'Date'        => $row['Date'],
                        'Title'       => $row['SubmoduleName'],
                        'Description' => $row['SubmoduleDescription'],
                        'Rate'        => (int)$row['Rate'],
                        'MaxRate'     => (int)$row['MaxRate'],
                    );
            }
        }
        $disMap['ModulesCount'] = $modulesCount;

        $disMap['SemesterRate'] = (int)$sRate;
        $disMap['SemesterMaxRate'] = (int)$sMaxRate;
        $disMap['ExamRate'] = (int)$examRate;
        $disMap['ExamMaxRate'] = (int)$examMaxRate;
        $disMap['BonusRate'] = (int)$bonusRate;
        $disMap['BonusMaxRate'] = (int)$bonusMaxRate;
        $disMap['ExtraRate'] = (int)$extraRate;

        return $disMap;
    }
}